import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.*;
import java.io.*;

public class Wordix extends MIDlet implements CommandListener  
{ 
    static Command CONTINUE_CMD = new Command("Continue", Command.SCREEN, 1);
    static Command START_CMD = new Command("Start", Command.SCREEN, 1);
    static Command STOP_CMD = new Command("Stop", Command.SCREEN, 1);
    static Command QUIT_CMD = new Command("Quit", Command.BACK, 3);
    static Command HELP_CMD = new Command("Help", Command.HELP, 4);
    static Command SETTINGS_CMD = new Command("Settings", Command.SCREEN, 2);
    static Command BACK_CMD = new Command("Back", Command.BACK, 1);
    static Command SAVE_CMD = new Command("Save", Command.OK, 1);

    static final String STORE_NAME = "WORDIX";
    static final int    MAX_DICTIONARY_SIZE = 24000;
    static final int    MAX_WORDS = 6000;

    String helpText = "Play tetris with words: create words from falling letters";

    static Image logoImage;

    static { 
        try { 
            logoImage = Image.createImage("/logo.png");
        } catch (java.io.IOException err) {}
    }


    Form         main;
    RecordStore  store;
    WordixCanvas game;
    Settings     settings;
    short[]      words;
    byte[]       dictionary;
    int          size;
    int          n_words;
    Random       rand;

    public Wordix() 
    {
        main = new Form("Wordix");
        rand = new Random();
        main.append(new ImageItem(null, logoImage, ImageItem.LAYOUT_CENTER, "Wordix"));
        main.addCommand(START_CMD);
        main.addCommand(HELP_CMD);
        main.addCommand(SETTINGS_CMD);
        main.addCommand(QUIT_CMD);
	try { 
            InputStream in = getClass().getResourceAsStream("/wordix.dic");
            dictionary = new byte[MAX_DICTIONARY_SIZE];
            size = in.read(dictionary);
            in.close();
        } catch (IOException x) {}
        words = new short[MAX_WORDS];
        byte[] text = dictionary;
        short[] w = words;
        int n = 0;
        for (int i = 0, s = size; i < s; i++) { 
            w[n++] = (short)i;
            while (text[i] != '\n') { 
                i += 1;
            }
        }
        n_words = n;
    }

    public int getColumnWordLength(byte[][] matrix, int x, int y) { 
        byte[] b = matrix[x];
        byte[] d = dictionary;
        int len = b.length;
        int l = 0;
        int r = n_words;
        int i, j, rr, m;

        do { 
            m = (l + r) >>> 1;
            j = words[m];
            rr = r;
            r = m;
            for (i = y; i < len; i++, j++) { 
                if (d[j] == '\n') { 
                    return i - y;
                }
                int diff = d[j] - b[i];
                if (diff < 0) {
                    l = m + 1;
                    r = rr;
                    break;
                } else if (diff > 0) { 
                    break;
                }
            }
        } while (l < r);
          
        return 0;
    }

    public int getRowWordLength(byte[][] matrix, int x, int y) { 
        byte[][] b = matrix;
        byte[] d = dictionary;
        int len = b.length;
        int l = 0;
        int r = n_words;
        int i, j, rr, m;


        do { 
            m = (l + r) >>> 1;
            j = words[m];
            rr = r;
            r = m;
            for (i = x; i < len; i++, j++) { 
                if (d[j] == '\n') { 
                    return i - x;
                }
                int diff = d[j] - b[i][y];
                if (diff < 0) {
                    l = m + 1;
                    r = rr;
                    break;
                } else if (diff > 0) { 
                    break;
                }
            }
        } while (l < r);
          
        return 0;
    }

    public byte nextLetter() { 
        byte b;
        while ((b = dictionary[(rand.nextInt() >>> 1) % size]) == '\n');
        return b;
    }
            
    protected void destroyApp(boolean unconditional) 
    {
        if (game != null) { 
            game.stop();
        }
        closeStore();
        main = null;
    }    

    protected  void pauseApp() 
    { 
        if (game != null) { 
            game.stop();
        }
    }

    protected void startApp() 
    {    
        Display display = Display.getDisplay(this);
	main.setCommandListener(this);
        display.setCurrent(main);
        if (store == null) { 
            openStore();
        }
    }


    private void openStore() {
        try { 
            try { 
                store = RecordStore.openRecordStore(STORE_NAME, false);
                loadSettings();
            } catch (RecordStoreNotFoundException x) { 
                store = RecordStore.openRecordStore(STORE_NAME, true);
                initializeSettings();
            }
        } catch(Exception x) { 
            settings = new Settings();
        }
    }

    private void closeStore() 
    {
        if (store != null) { 
            try { 
                store.closeRecordStore();
            } catch (Exception x) {}
        }
    }


    private void initializeSettings() {        
        settings = new Settings();
        byte[] record = settings.pack();
        try { 
            store.addRecord(record, 0, record.length);
        } catch (Exception x) {} 
    }
        
    private void loadSettings() {
        try { 
            settings = new Settings(store.getRecord(1));
        } catch (Exception x) {}
    }
        
    


    void saveSettings() { 
        byte[] record = settings.pack();
        try { 
            store.setRecord(1, record, 0, record.length);
        } catch (Exception x) {} 
    }

    public void commandAction(Command c, Displayable d) 
    {
        if (c == START_CMD) { 
            game = new WordixCanvas(this);
            game.start();
        } else if (c == HELP_CMD) {
            Alert help = new Alert("Help", helpText, null, AlertType.INFO);
            help.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(help, main);
        } else if (c == QUIT_CMD) { 
            destroyApp(true);
            notifyDestroyed();
        } else if (c == SETTINGS_CMD) { 
            new SettingsForm(this, main);
        }
    }
}



